require "active_support"
require "net/http"
require "uri"
require "yaml"

class Upupup::Worker

  STATUS = []
  STATUS << STATUS_UP = "up"
  STATUS << STATUS_DOWN = "down"

  def initialize
    @mutex = Mutex.new
    reload!
  end

  def reload!
    @task_configs = YAML.load_file("tasks.yaml")[:services]
    @task_configs.each do |task|
      task[:accept_codes] = Regexp.new task[:accept_codes] if task[:accept_codes]
      task[:url] = URI task[:url] if task[:url]
    end

    @task_configs.each do |task|
      puts "Loaded #{task[:name].to_s.underline}".green
    end

    if @threads
      stop_thread_loops
      start_thread_loops
    end
  end

  def status
    Upupup::Run.all
  end

  def stop_thread_loops
    if @threads
      @threads.each(&:kill)
      puts "Stopped threads".yellow
    end
  end

  def start_thread_loops
    @threads = @task_configs.map do |task_config|
      Thread.new do
        loop do
          puts "Run task #{task_config[:name].to_s.underline}".blue
          _execute_task_config(task_config)
          sleep task_config[:sleep]
        end
      end
    end
    puts "Started threads".green
    @threads
  end

  private def _execute_task_config(task_config)
    status = nil
    if task_config[:type] == "http"
      out = Net::HTTP.get_response(task_config[:url])
      is_code_valid = out.code.match(task_config[:accept_codes])
      status = is_code_valid ? STATUS_UP : STATUS_DOWN
    elsif task_config[:type] == "file"
      files = Dir.entries(task_config[:in_directory])
        .filter { |f| !f.start_with?(".") }
        .map { |f| File.join(task_config[:in_directory], f) }

      time_min = task_config[:ago][:amount].public_send(task_config[:ago][:unit]).ago
      file_found = files.find { |f| File.stat(f).send(task_config[:time]) >= time_min }
      status = file_found ? STATUS_UP : STATUS_DOWN
    else
      status = STATUS_DOWN
    end

    if status.nil?
      puts "Uknown config type #{task_config[:type].to_s.underline}".red
    else
      inject(name: task_config[:name], status: status)
    end

    self
  end

  RUN_LIMIT = 10

  private def inject(name:, status:)
    time = Time.now
    current = Upupup::Run.where(name: name).first ||
              Upupup::Run.new(name: name, status: STATUS_UP)

    current[:last_run] = time
    if current[:status] != status
      current[:status] = status
    end
    if status == STATUS_DOWN
      current[:last_down] = time
    elsif status == STATUS_UP
      current[:last_up] = time
    else
      puts "Error: bad status \"#{status}\"".red
    end
    @mutex.synchronize { current.save rescue nil }

    self
  end

end
